@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # gpodder.net API Client
-# Copyright (C) 2009-2010 Thomas Perl
+# Copyright (C) 2009-2013 Thomas Perl and the gPodder Team
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@ import mygpoclient
 
 from mygpoclient import util
 from mygpoclient import simple
+from mygpoclient import public
 
 # Additional error types for the advanced API client
 class InvalidResponse(Exception): pass
@@ -73,13 +74,13 @@ class PodcastDevice(object):
     def __init__(self, device_id, caption, type, subscriptions):
         # Check if the device type is valid
         if type not in self.VALID_TYPES:
-            raise ValueError('Invalid device type (see VALID_TYPES)')
+            raise ValueError('Invalid device type "%s" (see VALID_TYPES)' % type)
 
         # Check if subsciptions is a numeric value
         try:
             int(subscriptions)
         except:
-            raise ValueError('Subscription must be a numeric value')
+            raise ValueError('Subscription must be a numeric value but was %s' % subscriptions)
 
         self.device_id = device_id
         self.caption = caption
@@ -117,14 +118,14 @@ class EpisodeAction(object):
 
     The attribute "position" is only valid for "play" action types.
     """
-    VALID_ACTIONS = ('download', 'play', 'delete', 'new')
+    VALID_ACTIONS = ('download', 'play', 'delete', 'new', 'flattr')
 
     def __init__(self, podcast, episode, action,
             device=None, timestamp=None,
             started=None, position=None, total=None):
         # Check if the action is valid
         if action not in self.VALID_ACTIONS:
-            raise ValueError('Invalid action type (see VALID_TYPES)')
+            raise ValueError('Invalid action type "%s" (see VALID_ACTIONS)' % action)
 
         # Disallow play-only attributes for non-play actions
         if action != 'play':
@@ -138,7 +139,7 @@ class EpisodeAction(object):
         # Check the format of the timestamp value
         if timestamp is not None:
             if util.iso8601_to_datetime(timestamp) is None:
-                raise ValueError('Timestamp has to be in ISO 8601 format')
+                raise ValueError('Timestamp has to be in ISO 8601 format but was %s' % timestamp)
 
         # Check if we have a "position" value if we have started or total
         if position is None and (started is not None or total is not None):
@@ -149,21 +150,21 @@ class EpisodeAction(object):
             try:
                 started = int(started)
             except ValueError:
-                raise ValueError('Started must be an integer value (seconds)')
+                raise ValueError('Started must be an integer value (seconds) but was %s' % started)
 
         # Check that "position" is a number if it's set
         if position is not None:
             try:
                 position = int(position)
             except ValueError:
-                raise ValueError('Position must be an integer value (seconds)')
+                raise ValueError('Position must be an integer value (seconds) but was %s' % position)
 
         # Check that "total" is a number if it's set
         if total is not None:
             try:
                 total = int(total)
             except ValueError:
-                raise ValueError('Total must be an integer value (seconds)')
+                raise ValueError('Total must be an integer value (seconds) but was %s' % total)
 
         self.podcast = podcast
         self.episode = episode
@@ -195,7 +196,6 @@ class EpisodeAction(object):
 
         return d
 
-
 class MygPodderClient(simple.SimpleClient):
     """gpodder.net API Client
 
@@ -204,16 +204,19 @@ class MygPodderClient(simple.SimpleClient):
     for a smaller class that only implements the Simple API.
     """
 
+    @simple.needs_credentials
     def get_subscriptions(self, device):
         # Overloaded to accept PodcastDevice objects as arguments
         device = getattr(device, 'device_id', device)
         return simple.SimpleClient.get_subscriptions(self, device)
 
+    @simple.needs_credentials
     def put_subscriptions(self, device, urls):
         # Overloaded to accept PodcastDevice objects as arguments
         device = getattr(device, 'device_id', device)
         return simple.SimpleClient.put_subscriptions(self, device, urls)
 
+    @simple.needs_credentials
     def update_subscriptions(self, device_id, add_urls=[], remove_urls=[]):
         """Update the subscription list for a given device.
 
@@ -227,11 +230,11 @@ class MygPodderClient(simple.SimpleClient):
         """
         uri = self._locator.add_remove_subscriptions_uri(device_id)
 
-        if not all(isinstance(x, str) for x in add_urls):
-            raise ValueError('add_urls must be a list of strings')
+        if not all(isinstance(x, basestring) for x in add_urls):
+            raise ValueError('add_urls must be a list of strings but was %s' % add_urls)
 
-        if not all(isinstance(x, str) for x in remove_urls):
-            raise ValueError('remove_urls must be a list of strings')
+        if not all(isinstance(x, basestring) for x in remove_urls):
+            raise ValueError('remove_urls must be a list of strings but was %s' % remove_urls)
 
         data = {'add': add_urls, 'remove': remove_urls}
         response = self._client.POST(uri, data)
@@ -245,7 +248,7 @@ class MygPodderClient(simple.SimpleClient):
         try:
             since = int(response['timestamp'])
         except ValueError:
-            raise InvalidResponse('Invalid value for timestamp in response')
+            raise InvalidResponse('Invalid value %s for timestamp in response' % response['timestamp'])
 
         if 'update_urls' not in response:
             raise InvalidResponse('Response does not contain update_urls')
@@ -253,14 +256,15 @@ class MygPodderClient(simple.SimpleClient):
         try:
             update_urls = [(a, b) for a, b in response['update_urls']]
         except:
-            raise InvalidResponse('Invalid format of update_urls in response')
+            raise InvalidResponse('Invalid format of update_urls in response: %s' % response['update_urls'])
 
-        if not all(isinstance(a, str) and isinstance(b, str) \
-                for a, b in update_urls):
-            raise InvalidResponse('Invalid format of update_urls in response')
+        if not all(isinstance(a, basestring) and isinstance(b, basestring) \
+                    for a, b in update_urls):
+            raise InvalidResponse('Invalid format of update_urls in response: %s' % update_urls)
 
         return UpdateResult(update_urls, since)
 
+    @simple.needs_credentials
     def pull_subscriptions(self, device_id, since=None):
         """Downloads subscriptions since the time of the last update
 
@@ -287,19 +291,20 @@ class MygPodderClient(simple.SimpleClient):
         if 'timestamp' not in data:
             raise InvalidResponse('Timestamp missing from response')
 
-        if not all(isinstance(x, str) or isinstance(x, unicode) for x in data['add']):
-            raise InvalidResponse('Invalid value(s) in list of added podcasts')
+        if not all(isinstance(x, basestring) for x in data['add']):
+            raise InvalidResponse('Invalid value(s) in list of added podcasts: %s' % data['add'])
 
-        if not all(isinstance(x, str) or isinstance(x, unicode) for x in data['remove']):
-            raise InvalidResponse('Invalid value(s) in list of removed podcasts')
+        if not all(isinstance(x, basestring) for x in data['remove']):
+            raise InvalidResponse('Invalid value(s) in list of removed podcasts: %s' % data['remove'])
 
         try:
             since = int(data['timestamp'])
         except ValueError:
-            raise InvalidResponse('Timestamp has invalid format in response')
+            raise InvalidResponse('Timestamp has invalid format in response: %s' % data['timestamp'])
 
         return SubscriptionChanges(data['add'], data['remove'], since)
 
+    @simple.needs_credentials
     def upload_episode_actions(self, actions=[]):
         """Uploads a list of EpisodeAction objects to the server
 
@@ -318,10 +323,11 @@ class MygPodderClient(simple.SimpleClient):
         try:
             since = int(response['timestamp'])
         except ValueError:
-            raise InvalidResponse('Invalid value for timestamp in response')
+            raise InvalidResponse('Invalid value %s for timestamp in response' % response['timestamp'])
 
         return since
 
+    @simple.needs_credentials
     def download_episode_actions(self, since=None,
             podcast=None, device_id=None):
         """Downloads a list of EpisodeAction objects from the server
@@ -346,7 +352,7 @@ class MygPodderClient(simple.SimpleClient):
         try:
             since = int(data['timestamp'])
         except ValueError:
-            raise InvalidResponse('Invalid value for timestamp: ' +
+            raise InvalidResponse('Invalid value for timestamp: ' + 
                     data['timestamp'])
 
         dicts = data['actions']
@@ -357,6 +363,7 @@ class MygPodderClient(simple.SimpleClient):
 
         return EpisodeActionChanges(actions, since)
 
+    @simple.needs_credentials
     def update_device_settings(self, device_id, caption=None, type=None):
         """Update the description of a device on the server
 
@@ -378,6 +385,7 @@ class MygPodderClient(simple.SimpleClient):
             data['type'] = type
         return (self._client.POST(uri, data) is None)
 
+    @simple.needs_credentials
     def get_devices(self):
         """Returns a list of this user's PodcastDevice objects
 
@@ -395,4 +403,21 @@ class MygPodderClient(simple.SimpleClient):
         except KeyError:
             raise InvalidResponse('Missing keys in device list response')
 
-
+    def get_favorite_episodes(self):
+        """Returns a List of Episode Objects containing the Users
+        favorite Episodes"""
+        uri = self._locator.favorite_episodes_uri()
+        return [public.Episode.from_dict(d) for d in self._client.GET(uri)]
+    
+    def get_settings(self, type, scope_param1=None, scope_param2=None):
+        """Returns a Dictionary with the set settings for the type & specified scope"""
+        uri = self._locator.settings_uri(type, scope_param1, scope_param2)
+        return self._client.GET(uri)
+    
+    def set_settings(self, type, scope_param1, scope_param2, set={}, remove=[]):
+        """Returns a Dictionary with the set settings for the type & specified scope"""
+        uri = self._locator.settings_uri(type, scope_param1, scope_param2)
+        data = {}
+        data["set"] = set
+        data["remove"] = remove
+        return self._client.POST(uri, data)
